

function [out1,plot_data,ISbuffer,W,counters,cter,currents,temp] = HHneuron_Wallen1992(in1,dt,I_elec,i,neuron_flag,ISbuffer,W,counters,cter,currents,Rel,lastbuff,temp,nmda,kampa,fiveHT,TTX,nAMP,gCa,solver,t,KAHP,statevars,coeffis,network)


%% --- description --- %%
%
% This function solves the 15 coupled differential equations for the 
% Hodgkin-Huxley type neuron used in Ekeberg 1991 and Wallén 1992. It 
% is a 5 compartment model, consisting of an initial segment (similar 
% to the axon hillock), soma and 3 compartments representing various 
% distances from the soma along the dendritic tree. The membrane currents
% are Na+, K+, Ca2+ and a delayed Ca2+dependent K+ conductance. In 
% addition we have AMPA/Kainate and NMDA synaptic conductances. 
%



%% --- parameters --- %%



% IMPORTANT NOTE
%
% Normally all parameters should be allocated in arrays outside the main 
% time loop and the arrays should be passed to this script. 
% For educational purposes the parameters are included in this script, 
% so that the appropriate variable names appear in the equations below. 
% This way an equation will read for instance
%
%      K = (E_CaNMDA - Esoma) * rhoNMDA * MgBLOCKp; 
%
% or
%
%      ALPHm = Na_m_ALPH_A * (E0 - Na_m_ALPH_B) / ( 1 - exp((Na_m_ALPH_B - E0) / Na_m_ALPH_C) );
%
% instead of 
%
%      K = (E(11) - E(12)) * Paramter(43) * statevars(10);
%
% or
%
%      ALPHm = ChannelParameter(1) * (E(7) - ChannelParameter(2)) / ( 1 - exp((ChannelParameter(2) - E(7)) / ChannelParameter(3)) );     
%
% If the number of neurons and parameters is increased the penalty in terms of computation 
% time for this practice will become prohibitve and all parameters should 
% only be allocated once outside the main time loop, which is the correct
% programming practice.
 


% parameters for gating equations for the ion channels
% naming according to:
% IONTYPE_VARIABLE-NAME_OPENING/CLOSING-RATE_COEFFI
Na_m_ALPH_A   =    0.2;   % A coefficient   [1/mV 1/ms]
Na_m_ALPH_B   =    -45;   % B coefficient   [mV]
Na_m_ALPH_C   =      1;   % C coefficient   [mV]
Na_m_BETA_A   =   0.06;   % ...
Na_m_BETA_B   =    -54;
Na_m_BETA_C   =     20;
Na_h_ALPH_A   =   0.08;
Na_h_ALPH_B   =    -45;
Na_h_ALPH_C   =      1;
Na_h_BETA_A   =    0.4;
Na_h_BETA_B   =    -41;
Na_h_BETA_C   =      2;
K_n_ALPH_A    =   0.02;
K_n_ALPH_B    =    -45;
K_n_ALPH_C    =    0.8;
K_n_BETA_A    =  0.005;
K_n_BETA_B    =    -35;
K_n_BETA_C    =    0.4;
Ca_q_ALPH_A   =   0.08;
Ca_q_ALPH_B   =     15;
Ca_q_ALPH_C   =     11;
Ca_q_BETA_A   =  0.001;
Ca_q_BETA_B   =     15;
Ca_q_BETA_C   =    0.5;
NMDA_p_ALPH_A =    0.7;
NMDA_p_ALPH_C =     17;
NMDA_p_MG     =    1.8;   % for bath application
NMDA_p_BETA_A = 0.0056;   
NMDA_p_BETA_C =     17;   



% neuron sizes 
% conductances of ionic channels are given per square millimeter. Hence
% thay need to be multiplied by the size of the respective compartment
if  (neuron_flag == 1)         
    areaS   = 4*pi*0.01^2;     % EIN_L - excitatory interneuron
end                            
if  (neuron_flag == 2)         
    areaS   = 4*pi*0.04^2;     % LIN_L - lateral interneurons
end                            
if  (neuron_flag == 3)         
    areaS   = 4*pi*0.015^2;    % CCIN_L - cross-inhibitory interneurons
end                            
if  (neuron_flag == 5)         
    areaS   = 4*pi*0.01^2;     % EIN_R - excitatory interneuron
end                            
if  (neuron_flag == 6)         
    areaS   = 4*pi*0.04^2;     % LIN_R - lateral interneurons
end                            
if  (neuron_flag == 7)         
    areaS   = 4*pi*0.015^2;    % CCIN_R - cross-inhibitory interneurons
end                                                        
area_IS    =     0.1*areaS;    % area of the initial neuron segment
area_DEN   =       4*areaS;    % area of the dendritic compartments
                                                           
   

% other neuron parameters                            
Eleak      =           -70;         % resting potential                                    [mV]
CONcore    =     4.2*areaS;         % coupling condunctance between compartments          [müS]
CONcore_IS =   300*area_IS;         % ...                               
CONmE0     =  0.83*area_IS;         % ...
CONmSOMA   =    0.83*areaS;         % membrane/leak conductance for soma                  [müS]
CONmDEND   = 0.83*area_DEN;         % membrane/leak conductance for dendritic compartment [müS]
E_Na       =            50;         % reversal potential for Na                            [mV]
E_K        =           -80;         % reversal potential for K                             [mV]
CON_K      =      83*areaS;         % 83 max K channel condunctance                          [müS]
CON_K_IS   =   415*area_IS;         % 415 ...
E_Ca       =           150;         % reversal potential for Ca                            [mV]
E_CaNMDA   =            20;         % 
CON_Ca     =     gCa*areaS;         % max Ca channel cond. through membrane               [müS]

rhoAP      =    0.67*areaS*0.5;     % 0.67*areaS*0.8; Ca ion influx for ion pool in cell            [1/ms 1/mV]   0.3

rhoNMDA    =    0.0011;             % absolute value, not scaled with sneuron size (either synapase or bath receptor)

deltaAP    =          0.02*0.7;     % 0.02*areaS*0.6 Ca ion decay for ion pool in cell                  [1/ms]    0.5
deltaNMDA  =         0.002;         % Ca ion decay for NMDA related pool                 [1/ms]

CON_KdepCa_AHP  =    0.160;         % max KCa driven by membrane Ca                       [müS]
CON_KdepCa_NMDA =    0.040;         % max KCa for NMDA receptor                           [müS]
E_Syn_exc_K_AMPA     =   0;         % reversal potential excitatory synapses               [mV]
E_Syn_inh            = -85;         % reversal potential inhibitory synapse                [mV]
                               
% if ((neuron_flag == 2) || (neuron_flag == 6)) 
%     nAMP = nAMP*0.1;
% end


                               
% bath specific parameters
                                   
% TTX (Tetrodotoxin) is a drug that blocks action potential generating Na channels.
% In a physiological experiment the drug would be added to the chemical
% solution that a neuron is place in. To this "bath" various 
% pharmacological agent can be added, for instance TTX. 
% If "TTX" is not equal to 0, the Na channels are bloked in the 
% present model. 
if (TTX == 0)                        
    CON_Na     =    50*areaS;        % 250; 50 max Na channel cond. through membrane - GNa in paper      [müS]
    CON_Na_IS  = 835*area_IS;        % 750; 250 835*area_IS;        % ...
else                                 
    CON_Na     =           0;        % max Na channel cond. through membrane - GNa in paper      [müS]
    CON_Na_IS  =           0;        % ...
end

% Another addition to the "bath" can be the excitatory neurotransmitter
% NMDA (N-methyl-D-aspartate), which docks to NMDA sensitive glutamate 
% receptors. In the present model NMDA only docks at the soma.
if (nmda ~= 0)                                       % NMDA bath application
    if ((neuron_flag == 2) || (neuron_flag == 6))    % reduced for LINs
      CON_NMDA   = 7.5*areaS*nmda*0.6;
    else
      CON_NMDA   = 75*areaS*nmda*0.6;        
    end
else
    CON_NMDA = 0;
end

% Another addition to the "bath" can be the excitatory neurotransmitter
% AMPA (2-amino-3-(3-hydroxy-5-methyl-isoxazol-4-yl)propanoic acid), which
% docks to AMPA sensitive glutamate receptors. In the present model AMPA 
% docks at all compartments of the neuron model.
if (kampa ~= 0)                                      % KAMPA bath application, docks to all compartments
    if ((neuron_flag == 2) || (neuron_flag == 6))    % reduced for LINs
        CON_KAMPA_S  = 0.10*areaS*kampa;%*2;
        CON_KAMPA_1  = 0.10*area_DEN*kampa;%*2;
        CON_KAMPA_2  = 0.10*area_DEN*kampa;%*2;
        CON_KAMPA_3  = 0.10*area_DEN*kampa;%*2;     
    else
        CON_KAMPA_S  = 1.0*areaS*kampa;    
        CON_KAMPA_1  = 1.0*area_DEN*kampa;
        CON_KAMPA_2  = 1.0*area_DEN*kampa;
        CON_KAMPA_3  = 1.0*area_DEN*kampa;
    end
else
    CON_KAMPA_S  = 0;    
    CON_KAMPA_1  = 0;
    CON_KAMPA_2  = 0;
    CON_KAMPA_3  = 0;
end
E_NMDA     =       0; % reversal potential
E_KAMPA    =       0; % reversal potential



%% --- update runtime variables --- %%               

statevars(:,2) =             in1;          % old values / initial values
E0             = statevars(12,2);          % membrane potential for initial AP generating neuron compartment
Esoma          =  statevars(1,2);          % membrane potential for soma
Ecom1          =  statevars(2,2);          % potential for compartment 1
Ecom2          =  statevars(2,2);          % for compartment 2
Ecom3          =  statevars(2,2);          % for compartment 3
NaONm          =  statevars(5,2);          % activation gating variable of Na+ channels, m in paper
NaOFFh         =  statevars(6,2);          % inactivation gating variable of Na+ channels, h in paper
K_ONn          =  statevars(7,2);          % activation gating variable of K+ channels, n in paper
CaONq          =  statevars(8,2);          % activation gating variable of Ca2+ channels, q in paper
CaAHP          =  statevars(9,2);          % Ca2+ pool due to Ca-dependent K+ channels
MgBLOCKp       = statevars(10,2);          % variable for Mg2+ block of NMDA receptors, p in paper
CaNMDA         = statevars(11,2);          % Ca2+ pool due to opening of NMDA receptor channels
NaONm_IS       = statevars(13,2);          % activation gating variable of Na+ channels, m in paper
NaOFFh_IS      = statevars(14,2);          % inactivation gating variable of Na+ channels, h in paper
K_ONn_IS       = statevars(15,2);          % activation gating variable of K+ channels, n in paper


                                           
%% --- update compartment potentials, update Ca and resolve synaptic communication --- %%

% choices for the implicit solver for integrating multicompartment models
% according to Chapter 6.6 in "Theoretical Neuroscience" by Dayan and Abbott
% z = 0.5;    % Crank-Nicholson method
z = 1;        % reverse Euler method


% in even time time steps resolve synaptic communication, update membrane
% potential and calcium concentrations
if (mod(i,2) == 0)
    
    % --- resolve synaptic communication and update synaptic currents/conductances
    [currents, ISbuffer, counters, temp, W] = Syn_Trans(statevars, ISbuffer, W, counters, neuron_flag, currents, Rel, dt, lastbuff, temp, nAMP, network);
    
    inh_index_1 = Rel(1,2,neuron_flag);
    inh_index_2 = Rel(2,2,neuron_flag);
    exc_index_1 = Rel(1,1,neuron_flag);
    
    if (inh_index_1 ~= 0)
        INH1weight = W(inh_index_1,neuron_flag,1);
        INH1cond   = INH1weight * W(inh_index_1,neuron_flag,8);
    else
        INH1weight = 0;
        INH1cond   = 0;
    end
    if (inh_index_2 ~= 0)
        INH2weight = W(inh_index_2,neuron_flag,1);
        INH2cond   = INH2weight * W(inh_index_2,neuron_flag,2);
    else
        INH2weight = 0;
        INH2cond   = 0;
    end
    if (exc_index_1 ~= 0)
        EXC1weight = W(exc_index_1,neuron_flag,1);
        EXC1cond   = EXC1weight * W(exc_index_1,neuron_flag,3);
    else
        EXC1weight = 0;
        EXC1cond   = 0;
    end
    
    CON_NMDA_from_spike = EXC1weight * W(exc_index_1,neuron_flag,4);          % delayed and calculated in Syn_Trans
    
    CON_NMDA_tot = CON_NMDA_from_spike + CON_NMDA;
    
    %rhoNMDA   = 0.0011*(CON_NMDA + CON_NMDA_from_spike)*1.0;                 % Ca ion influx rate NMDA gated, see table 2 in Wallen1992 --- used below in update of concentartions
    
    g_m = zeros(6,8);
    Ei  = zeros(6,8);
    
    % conductances and equilibrium potentials
    % initial segment
    g_m(1,1) = CON_Na_IS * NaONm_IS^3 * NaOFFh_IS;
    g_m(1,2) = CON_K_IS * K_ONn_IS^4;
    g_m(1,3) = CONmE0;
    Ei(1,1)  = E_Na;
    Ei(1,2)  = E_K;
    Ei(1,3)  = Eleak;
    % soma
    g_m(2,1) = CON_Na * NaONm^3 * NaOFFh;
    g_m(2,2) = CON_K * K_ONn^4;
    g_m(2,3) = CON_Ca * CaONq^5;
    
    g_m(2,4) = (1-fiveHT) * KAHP * CON_KdepCa_AHP * CaAHP;  % serotonin, 5HT, acts here on the KCa conductance
    
    g_m(2,5) = CON_NMDA_tot * MgBLOCKp;
    
    g_m(2,6) = CON_KAMPA_S;
    g_m(2,7) = CONmSOMA;
    
    g_m(2,8) = KAHP * CON_KdepCa_NMDA * CaNMDA;
    
    Ei(2,1)  = E_Na;
    Ei(2,2)  = E_K;
    Ei(2,3)  = E_Ca;
    Ei(2,4)  = E_K;
    Ei(2,5)  = E_NMDA;
    Ei(2,6)  = E_KAMPA;
    Ei(2,7)  = Eleak;
    
    Ei(2,8) = E_K;
    
    % dendritic C1
    g_m(3,1) = CON_KAMPA_1;
    g_m(3,2) = INH1cond;
    g_m(3,3) = INH2cond;
    g_m(3,4) = W(1,neuron_flag,7)*nAMP;
    g_m(3,5) = CONmDEND;
    Ei(3,1)  = E_KAMPA;
    Ei(3,2)  = E_Syn_inh;
    Ei(3,3)  = E_Syn_inh;
    Ei(3,4)  = E_Syn_exc_K_AMPA;
    Ei(3,5)  = Eleak;
    % dendritic C2
    g_m(4,1) = CON_KAMPA_2;
    g_m(4,2) = EXC1cond;
    g_m(4,3) = CONmDEND;
    Ei(4,1)  = E_KAMPA;
    Ei(4,2)  = E_Syn_exc_K_AMPA;
    Ei(4,3)  = Eleak;
    % dendritic C3
    g_m(5,1) = CON_KAMPA_3;
    g_m(5,2) = CONmDEND;
    Ei(5,1)  = E_KAMPA;
    Ei(5,2)  = Eleak;
    
    
    new = implicit_solver(neuron_flag, statevars, g_m, Ei, z, dt, I_elec, t);
    
    
    statevars(12,1) = new(12,1);
    statevars(1,1)  =  new(1,1);
    statevars(2,1)  =  new(2,1);
    statevars(3,1)  =  new(3,1);
    statevars(4,1)  =  new(4,1);
    
    %ISbuffer(2,neuron_flag) = new(12,1);
    
    % CaAHP eq - Ca pool - row 9 of statevars
    coeffis(9,1)   = (E_Ca - Esoma)*rhoAP*CaONq^5;                                        % K for DEQ9    "DEQ" refers to differential equation, the number to the equation in Wallen er al. 1992
    coeffis(9,2)   = -deltaAP;                                                            % L for DEQ9
    Yold           = statevars(9,2);                                                      % = CaAHP
    Ynew           = ODEsolver(Yold,dt,coeffis(9,1),coeffis(9,2),solver);
    statevars(9,1) = Ynew;
    % CaNMDA eq - Ca pool - row 11 of statevars
    coeffis(11,1)  = (E_CaNMDA - Esoma)*rhoNMDA*MgBLOCKp*CON_NMDA_tot;                    % K for DEQ11
    coeffis(11,2)  = -deltaNMDA;                                                          % L for DEQ11
    Yold  = statevars(11,2);                                                              % = CaNMDA
    Ynew = ODEsolver(Yold,dt,coeffis(11,1),coeffis(11,2),solver);
    statevars(11,1) = Ynew;
    
    statevars(5,1)  =  statevars(5,2);       % old values carried over in uneven time steps
    statevars(6,1)  =  statevars(6,2);       % due to split update
    statevars(7,1)  =  statevars(7,2);
    statevars(8,1)  =  statevars(8,2);
    statevars(10,1) = statevars(10,2);
    statevars(13,1) = statevars(13,2);
    statevars(14,1) = statevars(14,2);
    statevars(15,1) = statevars(15,2);

end


% in uneven timesteps update channel kinetics
if (mod(i,2) == 1)
    
    % m eq for IS - Na activation - row 13 of statevars
    ALPHm           = Na_m_ALPH_A * (E0 - Na_m_ALPH_B) / ( 1 - exp((Na_m_ALPH_B - E0) / Na_m_ALPH_C) );
    BETAm           = Na_m_BETA_A * (Na_m_BETA_B - E0) / ( 1 - exp((E0 - Na_m_BETA_B) / Na_m_BETA_C) );
    coeffis(13,1)   =          ALPHm;                                                     % K for DEQ13
    coeffis(13,2)   = -ALPHm - BETAm;                                                     % L for DEQ13
    Yold            = statevars(13,2);
    Ynew            = ODEsolver(Yold,dt,coeffis(13,1),coeffis(13,2),solver);
    statevars(13,1) = Ynew;
    
    ISbuffer(2,neuron_flag) = Ynew;
    
    % h eq for IS - Na inactivation - row 14 of statevars
    ALPHh           = Na_h_ALPH_A * (Na_h_ALPH_B - E0) / ( 1 - exp((E0 - Na_h_ALPH_B) / Na_h_ALPH_C) );
    BETAh           = Na_h_BETA_A / ( 1 + exp((Na_h_BETA_B - E0) / Na_h_BETA_C) );
    coeffis(14,1)   =          ALPHh;                                                     % K for DEQ14
    coeffis(14,2)   = -ALPHh - BETAh;                                                     % L for DEQ14
    Yold            = statevars(14,2);
    Ynew            = ODEsolver(Yold,dt,coeffis(14,1),coeffis(14,2),solver);
    statevars(14,1) = Ynew;
    
    % n eq for IS - K activation - row 15 of statevars
    ALPHn           = K_n_ALPH_A * (E0 - K_n_ALPH_B) / ( 1 - exp((K_n_ALPH_B - E0) / K_n_ALPH_C) );
    BETAn           = K_n_BETA_A * (K_n_BETA_B - E0) / ( 1 - exp((E0 - K_n_BETA_B) / K_n_BETA_C) );
    coeffis(15,1)   =          ALPHn;                                                     % K for DEQ15
    coeffis(15,2)   = -ALPHn - BETAn;                                                     % L for DEQ15
    Yold            = statevars(15,2);
    Ynew            = ODEsolver(Yold,dt,coeffis(15,1),coeffis(15,2),solver);
    statevars(15,1) = Ynew;
    
    % m eq - Na activation - row 5 of statevars
    ALPHm           = Na_m_ALPH_A * (Esoma - Na_m_ALPH_B) / ( 1 - exp((Na_m_ALPH_B - Esoma) / Na_m_ALPH_C) );
    BETAm           = Na_m_BETA_A * (Na_m_BETA_B - Esoma) / ( 1 - exp((Esoma - Na_m_BETA_B) / Na_m_BETA_C) );
    coeffis(5,1)    =          ALPHm;                                                     % K for DEQ5
    coeffis(5,2)    = -ALPHm - BETAm;                                                     % L for DEQ5
    Yold            = statevars(5,2);
    Ynew            = ODEsolver(Yold,dt,coeffis(5,1),coeffis(5,2),solver);
    statevars(5,1)  = Ynew;
    
    % h eq - Na inactivation - row 6 of statevars
    ALPHh           = Na_h_ALPH_A * (Na_h_ALPH_B - Esoma) / ( 1 - exp((Esoma - Na_h_ALPH_B) / Na_h_ALPH_C) );
    BETAh           = Na_h_BETA_A / ( 1 + exp((Na_h_BETA_B - Esoma) / Na_h_BETA_C) );
    coeffis(6,1)    =          ALPHh;                                                     % K for DEQ6
    coeffis(6,2)    = -ALPHh - BETAh;                                                     % L for DEQ6
    Yold            = statevars(6,2);
    Ynew            = ODEsolver(Yold,dt,coeffis(6,1),coeffis(6,2),solver);
    statevars(6,1)  = Ynew;
    
    % n state variable - K activation
    ALPHn           = K_n_ALPH_A * (Esoma - K_n_ALPH_B) / ( 1 - exp((K_n_ALPH_B - Esoma) / K_n_ALPH_C) );
    BETAn           = K_n_BETA_A * (K_n_BETA_B - Esoma) / ( 1 - exp((Esoma - K_n_BETA_B) / K_n_BETA_C) );
    coeffis(7,1)    =          ALPHn;                                                     % K for DEQ7
    coeffis(7,2)    = -ALPHn - BETAn;                                                     % L for DEQ7
    Yold            = statevars(7,2);
    Ynew            = ODEsolver(Yold,dt,coeffis(7,1),coeffis(7,2),solver);
    statevars(7,1)  = Ynew;
    
    % q eq - Ca activation - row 8 of statevars
    ALPHq           = Ca_q_ALPH_A * (Esoma - Ca_q_ALPH_B) / ( 1 - exp((Ca_q_ALPH_B - Esoma) / Ca_q_ALPH_C) );
    BETAq           = Ca_q_BETA_A * (Ca_q_BETA_B - Esoma) / ( 1 - exp((Esoma - Ca_q_BETA_B) / Ca_q_BETA_C) );
    coeffis(8,1)    =          ALPHq;                                                     % K for DEQ8
    coeffis(8,2)    = -ALPHq - BETAq;                                                     % L for DEQ8
    Yold            = statevars(8,2);
    Ynew            = ODEsolver(Yold,dt,coeffis(8,1),coeffis(8,2),solver);
    statevars(8,1)  = Ynew;
    
    % p eq - Mg2+ Block - row 10 of statevars
    ALPHp = NMDA_p_ALPH_A * exp(Esoma/NMDA_p_ALPH_C);
    BETAp = NMDA_p_BETA_A * NMDA_p_MG * exp(-Esoma/NMDA_p_BETA_C);
    coeffis(10,1)     =          ALPHp;                                                   % K for DEQ10
    coeffis(10,2)     = -ALPHp - BETAp;                                                   % L for DEQ10
    Yold  = statevars(10,2);
    Ynew            = ODEsolver(Yold,dt,coeffis(10,1),coeffis(10,2),solver);
    statevars(10,1) = Ynew;
    
    statevars(1,1)  =  statevars(1,2); % old values carried over in uneven time steps
    statevars(2,1)  =  statevars(2,2); % due to split update
    statevars(3,1)  =  statevars(3,2);
    statevars(4,1)  =  statevars(4,2);
    statevars(12,1) = statevars(12,2);
    statevars(9,1)  =  statevars(9,2);
    statevars(11,1) = statevars(11,2);
    
end


    
%% --- prepare data for plotting ---

if (mod(i,2) == 1)
    plot_data = 0;
end

if (mod(i,2) == 0)
    
    % --- initial segment currents
    I_Na_IS  = (E_Na - E0) * CON_Na_IS * NaONm_IS^3 * NaOFFh_IS;
    I_K_IS   = (E_K - E0) * CON_K_IS * K_ONn_IS^4;
    % I_Na_IS  = (E_Na - E0) * CON_Na_IS * NaONm^3 * NaOFFh;
    % I_K_IS   = (E_K - E0) * CON_K_IS * K_ONn^4;
    % --- soma currents
    I_Na     = (E_Na - Esoma) * CON_Na * NaONm^3 * NaOFFh;
    I_K      = (E_K - Esoma) * CON_K * K_ONn^4;
    I_Ca     = (E_Ca - Esoma) * CON_Ca * CaONq^5;      % effectively zero due to Con_Ca = 0
    %I_KdepCa = (E_K - Esoma) * (CON_KdepCa_AHP * CaAHP + CON_KdepCa_NMDA * CaNMDA);
    I_KdepCa = (E_K - Esoma) * (CON_KdepCa_AHP * statevars(16,1) + CON_KdepCa_NMDA * CaNMDA);
    I_NMDA   = (CON_NMDA + CON_NMDA_from_spike) * MgBLOCKp * (E_NMDA - Esoma); % due either to synaptic transmission (see line 155) or bath or both - goes into soma
    % --- K/AMPA bath currents - no if necessary since above set to conductance set to zero if no bath
    I_KAMPA_s = CON_KAMPA_S *(E_KAMPA - Esoma); %currents(neuron_flag,8) + currents(neuron_flag,11);
    I_KAMPA_1 = CON_KAMPA_1 *(E_KAMPA - Ecom1); %currents(neuron_flag,9) + currents(neuron_flag,12);
    I_KAMPA_2 = CON_KAMPA_2 *(E_KAMPA - Ecom2);
    I_KAMPA_3 = CON_KAMPA_3 *(E_KAMPA - Ecom3); %currents(neuron_flag,10) + currents(neuron_flag,13);    
    % --- inhibitory and excitatory currents to comp1 and comp2
    I_Syn_inh1      = INH1weight * (E_Syn_inh - Ecom1) * W(inh_index_1,neuron_flag,8);
    I_Syn_inh2      = INH2weight * (E_Syn_inh - Ecom1) * W(inh_index_1,neuron_flag,2);
    I_Syn_exc_KAMPA = EXC1weight * (E_Syn_exc_K_AMPA - Ecom2) * W(exc_index_1,neuron_flag,3);    
    % --- update currents for compartments ---
    I_IS    = (I_Na_IS + I_K_IS)*5;                                               % *5 because of 5 times higher channel density in initial segment IS
    Isoma   = I_Na +  I_K  +  I_Ca  +  I_KdepCa + I_elec + I_NMDA + I_KAMPA_s;    % currents(neuron_flag,4 and 6) = NMDA current on soma
    Icomp1  = I_Syn_inh1 + I_Syn_inh2 + currents(neuron_flag,7) + I_KAMPA_1;      % inhibitory currents and noise and KAMPA bath dock on compartment 1
    Icomp2  = I_Syn_exc_KAMPA + I_KAMPA_2;                                        % excitatory current docks on comp2
    Icomp3  = I_KAMPA_3;
    
    plot_data = zeros(32,1,1);
    
    plot_data(1,1,1)  = statevars(12,1); % E initial segment 
    plot_data(2,1,1)  = statevars(1,1);  % Esoma 
    plot_data(3,1,1)  = statevars(2,1);  % Ecomp1 
    plot_data(4,1,1)  = statevars(3,1);  % Ecomp2 
    plot_data(5,1,1)  = statevars(4,1);  % Ecomp3     
    plot_data(6,1,1)  = statevars(9,1);  % Ca pool for AHP 
    plot_data(7,1,1)  = statevars(5,1);  % m - Na activation 
    plot_data(8,1,1)  = statevars(6,1);  % h - Na inactivation 
    plot_data(9,1,1)  = statevars(7,1);  % n - K activation 
    plot_data(10,1,1) = statevars(8,1);  % q - Ca activation    
    plot_data(11,1,1) = statevars(10,1); % p - MgBlock 
    plot_data(12,1,1) = statevars(11,1); % Ca pool for NMDA
    plot_data(13,1,1) = statevars(13,1); % IS Na activation
    plot_data(14,1,1) = statevars(14,1); % IS Na inactivation
    plot_data(15,1,1) = statevars(15,1); % IS K activation
    plot_data(16,1,1) = CON_Na*statevars(5,1)^3*statevars(6,1);      % Na cunductance soma as fct of time 
    plot_data(17,1,1) = CON_K*statevars(7,1)^4;                      % K cundactance soma as fct of time 
    plot_data(18,1,1) = CON_Na_IS*statevars(5,1)^3*statevars(6,1);   % Na cunductance IS as fct of time 
    plot_data(19,1,1) = CON_K_IS*statevars(7,1)^4;                   % K cundactance IS as fct of time     
    plot_data(20,1,1) = CON_NMDA_from_spike;                         % delayed and calculated in Syn_Trans
    plot_data(21,1,1) = CON_NMDA;                                    % from bath
    plot_data(22,1,1) = rhoNMDA;                                     % Ca ion influx rate NMDA gated, see table 2 in Wallen1992 --- used below in update of concentartions
    plot_data(23,1,1) = I_Na_IS;                                     % to IS
    plot_data(24,1,1) = I_K_IS;
    plot_data(25,1,1) = I_Na;                                        % to soma
    plot_data(26,1,1) = I_K;
    plot_data(27,1,1) = I_Ca;  
    plot_data(28,1,1) = I_KdepCa;
    plot_data(29,1,1) = I_NMDA;                                     % due either to synaptic transmission (see line 155) or bath or both - goes into soma
    plot_data(30,1,1) = I_KAMPA_s;                                  % form bath
    plot_data(31,1,1) = I_KAMPA_1; 
    plot_data(32,1,1) = I_KAMPA_2;
    plot_data(33,1,1) = I_KAMPA_3; 
    plot_data(34,1,1) = I_Syn_inh1;
    plot_data(35,1,1) = I_Syn_inh2;
    plot_data(36,1,1) = I_Syn_exc_KAMPA;                            % NMDA exc current is part of in I_NMDA    
    plot_data(37,1,1) = I_IS + CONmE0 * (Eleak - E0) + CONcore_IS*(Esoma-E0);
    plot_data(38,1,1) = Isoma + CONmSOMA * (Eleak - Esoma) + CONcore*(Ecom1-Esoma) + CONcore_IS*(E0-Esoma);
    plot_data(39,1,1) = Icomp1 + CONmDEND * (Eleak - Ecom1) + CONcore*(Ecom2-Ecom1) + CONcore*(Esoma-Ecom1);
    plot_data(40,1,1) = Icomp2 + CONmDEND * (Eleak - Ecom2) + CONcore*(Ecom3-Ecom2) + CONcore*(Ecom1-Ecom2);
    plot_data(41,1,1) = Icomp3 + CONmDEND * (Eleak - Ecom3) + CONcore*(Ecom2-Ecom3); 
    plot_data(42,1,1) = currents(neuron_flag,7);             % I_noise current due to PProcess
    plot_data(43,1,1) = W(1,1,4)*W(1,1,1);                   % NMDA conductace EIN to EIN from spikes      
    plot_data(44,1,1) = W(1,3,4)*W(1,3,1);                   % NMDA conductace EIN to CCIN  ...
    plot_data(45,1,1) = W(1,1,3)*W(1,1,1);                   % KAMPA conductace EIN to EIN     ... 
    plot_data(46,1,1) = W(1,3,3)*W(1,3,1);                   % KAMPA conductace EIN to CCIN  ...
    plot_data(47,1,1) = 0;                                   % timeslot, filled in metascript
    plot_data(48,1,1) = I_elec;

end



%%


out1  = statevars(:,1);



end

