
function [currents, ISbuffer, counters, temp, W] = Syn_Trans(statevars, ISbuffer, W, counters, neuron_flag, currents, Rel, dt, lastbuff, temp, noiseAMP, network)

%% --- description --- %%
%
% The implementation of synaptic transmission given below is by no means
% the most efficient one. The current implementation has been chosen readability. 
% A more efficient implementation in MatLab maybe 
% achieved by using locigal indexing of arrays. Furthermore the distinction 
% between the "W" and "Rel" matrices is not strictly necessary. 

% Also - as in the case of the parameter allocation in the script 
% "HHneuron_Wallen1992.m" - normally parameters that do not change 
% during the simulation should be allocated outside the
% main time loop. 

% In the current model a neuron can
% have at most 1 excitatory input (from the ipsilateral EIN) and at most 2
% inhibitory inputs (one from the ipsilateral LIN and one from the
% contralateral CCIN). These inputs are resolved separatly here.
%


%% --- synaptic parameters --- %

CON_Syn_inh          =  0.08;  % conductance after spike at inhibitory synapses - must be multiplied by weight        [müS]
CON_Syn_exc_K_AMPA   = 0.008;  % cond. at exc. synapses that targets K/AMPA receptors after spike  - ...              [müS]
CON_Syn_exc_NMDA     = 0.004;  % ...
E_Syn_exc_K_AMPA     =     0;  % reversal potential for synaptic current for excitatory synapses
E_Syn_exc_NMDA       =     0;  % NMDA gated current equilibrium potential
E_Syn_inh            =   -85;  % reversal potential for synaptic current for inhibitory synapses
tau_decay_inh        =    20;  % in ms
tau_decay_exc_K_AMPA =    20;  % ...
tau_decay_exc_NMDA   =   100;  % ...
thresh               =   0.8;  % treshold for AP detection (activation of Na Channels)



%% --- preliminaries ---

Ecomp1  =  statevars(2,2);
Ecomp2  =  statevars(3,2);
Esoma   =  statevars(1,2);
MgBlock = statevars(10,2);

exc_index_1 = Rel(1,1,neuron_flag);
%exc_index_2 = Rel(2,1,neuron_flag);
inh_index_1 = Rel(1,2,neuron_flag);
inh_index_2 = Rel(2,2,neuron_flag);



%% --- 1st inhibitory input --- %%

if (inh_index_1 ~= 0)
    
    if ((neuron_flag == 3 ) || (neuron_flag == 7))
        tau_decay_inh = 40;
    end
    
    weight          = W(inh_index_1,neuron_flag,1);
    E0_delayed      = ISbuffer(1/dt+1,inh_index_1);
    E0_before       = ISbuffer(1/dt+2,inh_index_1);
    
    if ((E0_delayed <= thresh) && (W(inh_index_1,neuron_flag,8) ~= 0))                             % no more AP and no new AP, only residual conductance
        CON_temp2 = W(inh_index_1,neuron_flag,8) * exp(-(dt/tau_decay_inh));                       % conductance decays further
        W(inh_index_1,neuron_flag,8)  = CON_temp2;                                                 % memorize for next timestep
    end
    if ((E0_delayed > thresh) && (W(inh_index_1,neuron_flag,8) ~= 0) && (E0_before > thresh))      % within AP
        CON_temp2 = W(inh_index_1,neuron_flag,8) * exp(-(dt/tau_decay_inh));                       % conductance decays further
        W(inh_index_1,neuron_flag,8)  = CON_temp2;                                                 % memorize for next timestep
    end
    if ((E0_delayed > thresh) && (W(inh_index_1,neuron_flag,8) ~= 0) && (E0_before <= thresh))     % no more AP, new AP arrives and residual conductance is present
        CON_temp2 = CON_Syn_inh;                                                                   % new conductance upon spike + residual decayed conductance
        W(inh_index_1,neuron_flag,8)  = CON_temp2;                                                 % memorize for next timestep
    end
    if ((E0_delayed <= thresh) && (W(inh_index_1,neuron_flag,8) == 0))     % no AP and no residual conductance - ATTENTION: the == 0 if must come after the ~=, otherwise two ifs are evaluated
        CON_temp2 = W(inh_index_1,neuron_flag,8);                          % => CON_temp2 = 0, initially the case
    end
    if ((E0_delayed > thresh) && (W(inh_index_1,neuron_flag,8) == 0))      % new AP arrives and no residual conductance
        CON_temp2 = CON_Syn_inh;                                           % synaptic conductance CON_temp2 is full conductance after spike
        W(inh_index_1,neuron_flag,8) = CON_temp2;                          % memorize for next timestep
    end
    
    I_Syn_inh                      = weight * (E_Syn_inh - Ecomp1) * CON_temp2;
    ISbuffer(lastbuff,inh_index_1) = 0;
    currents(neuron_flag,1)        = I_Syn_inh;
    temp(1,1)                      = I_Syn_inh;
    
end



%% --- 2nd inhibitory input --- %%

if (inh_index_2 ~= 0)
    
    tau_decay_inh = 20;
    
    weight          = W(inh_index_2,neuron_flag,1);
    E0_delayed      = ISbuffer(1/dt+1,inh_index_2);
    E0_before       = ISbuffer(1/dt+2,inh_index_2);
    
    if ((E0_delayed <= thresh) && (W(inh_index_2,neuron_flag,2) ~= 0))                             % no more AP and no new AP, only residual conductance
        CON_temp1 = W(inh_index_2,neuron_flag,2) * exp(-(dt/tau_decay_inh));                       % conductance decays further
        W(inh_index_2,neuron_flag,2)  = CON_temp1;                                                 % memorize for next timestep
    end
    if ((E0_delayed > thresh) && (W(inh_index_2,neuron_flag,2) ~= 0) && (E0_before > thresh))      % within AP
        CON_temp1 = W(inh_index_2,neuron_flag,2) * exp(-(dt/tau_decay_inh));                       % conductance decays further
        W(inh_index_2,neuron_flag,2)  = CON_temp1;                                                 % memorize for next timestep
    end
    if ((E0_delayed > thresh) && (W(inh_index_2,neuron_flag,2) ~= 0) && (E0_before <= thresh))     % no more AP, new AP arrives and residual conductance is present
        CON_temp1 = CON_Syn_inh;                                                                   % new conductance upon spike + residual decayed conductance
        W(inh_index_2,neuron_flag,2)  = CON_temp1;                                                 % memorize for next timestep
    end
    if ((E0_delayed <= thresh) && (W(inh_index_2,neuron_flag,2) == 0))     % no AP and no residual conductance - ATTENTION: the == 0 if must come after the ~=, otherwise two ifs are evaluated
        CON_temp1 = W(inh_index_2,neuron_flag,2);                          % => CON_temp1 = 0, initially the case
    end
    if ((E0_delayed > thresh) && (W(inh_index_2,neuron_flag,2) == 0))      % new AP arrives and no residual conductance
        CON_temp1 = CON_Syn_inh;                                           % synaptic conductance CON_temp1 is full conductance after spike
        W(inh_index_2,neuron_flag,2) = CON_temp1;                          % memorize for next timestep
    end
    
    I_Syn_inh                      = weight * (E_Syn_inh - Ecomp1) * CON_temp1;
    ISbuffer(lastbuff,inh_index_2) = 0;
    currents(neuron_flag,2)        = I_Syn_inh;
    temp(2,1)                      = I_Syn_inh;
    
end



%% --- 1st excitatory input --- %%

if (exc_index_1 ~= 0)
    
    % NMDA
    weight          = W(exc_index_1,neuron_flag,1);
    E0_delayed      = ISbuffer(1/dt+1,exc_index_1);
    E0_before       = ISbuffer(1/dt+2,exc_index_1);
    
    if ((E0_delayed <= thresh) && (W(exc_index_1,neuron_flag,4) ~= 0))                                      % no more AP and no new AP, only residual conductance
        CON_temp_NMDA = W(exc_index_1,neuron_flag,4) * exp(-(dt/tau_decay_exc_NMDA));                       % conductance decays further
        W(exc_index_1,neuron_flag,4)  = CON_temp_NMDA;                                                      % memorize for next timestep
    end
    if ((E0_delayed > thresh) && (W(exc_index_1,neuron_flag,4) ~= 0) && (E0_before > thresh))               % within AP
        CON_temp_NMDA = W(exc_index_1,neuron_flag,4) * exp(-(dt/tau_decay_exc_NMDA));                       % conductance decays further
        W(exc_index_1,neuron_flag,4)  = CON_temp_NMDA;                                                      % memorize for next timestep
    end
    if ((E0_delayed > thresh) && (W(exc_index_1,neuron_flag,4) ~= 0) && (E0_before <= thresh))              % no more AP, new AP arrives and residual conductance is present
        CON_temp_NMDA = CON_Syn_exc_NMDA;% +  W(exc_index_1,neuron_flag,4) * exp(-(dt/tau_decay_exc_NMDA)); % new conductance upon spike + residual decayed conductance
        W(exc_index_1,neuron_flag,4)  = CON_temp_NMDA;                                                      % memorize for next timestep
    end
    if ((E0_delayed <= thresh) && (W(exc_index_1,neuron_flag,4) == 0))   % no AP and no residual conductance - ATTENTION: the == 0 if must come after the ~=, otherwise two ifs are evaluated
        CON_temp_NMDA = W(exc_index_1,neuron_flag,4);                    % => CON_temp = 0, initially the case
    end
    if ((E0_delayed > thresh) && (W(exc_index_1,neuron_flag,4) == 0))    %&& ((E0_before <= thresh) || (E0_before > thresh) ))  % new AP arrives and no residual conductance
        CON_temp_NMDA = CON_Syn_exc_NMDA;                                % synaptic conductance CON_temp is full conductance after spike
        W(exc_index_1,neuron_flag,4) = CON_temp_NMDA;                    % memorize for next timestep
    end
    
    
    % Kainate/AMPA
    if ((E0_delayed <= thresh) && (W(exc_index_1,neuron_flag,3) ~= 0))                                           % no more AP and no new AP, only residual conductance
        CON_temp_KAMPA = W(exc_index_1,neuron_flag,3) * exp(-(dt/tau_decay_exc_K_AMPA));                         % conductance decays further
        W(exc_index_1,neuron_flag,3)  = CON_temp_KAMPA;                                                          % memorize for next timestep
    end
    if ((E0_delayed > thresh) && (W(exc_index_1,neuron_flag,3) ~= 0) &&  (E0_before > thresh))                   % within AP
        CON_temp_KAMPA = W(exc_index_1,neuron_flag,3) * exp(-(dt/tau_decay_exc_K_AMPA));                         % conductance decays further
        W(exc_index_1,neuron_flag,3)  = CON_temp_KAMPA;                                                          % memorize for next timestep
    end
    if ((E0_delayed > thresh) && (W(exc_index_1,neuron_flag,3) ~= 0) && (E0_before <= thresh))                   % no more AP, new AP arrives and residual conductance is present)
        CON_temp_KAMPA = CON_Syn_exc_K_AMPA +  W(exc_index_1,neuron_flag,3) * exp(-(dt/tau_decay_exc_K_AMPA));   % new conductance upon spike + residual decayed conductance
        W(exc_index_1,neuron_flag,3)  = CON_temp_KAMPA;                                                          % memorize for next timestep
    end
    if ((E0_delayed <= thresh) && (W(exc_index_1,neuron_flag,3) == 0))   % no AP and no residual conductance
        CON_temp_KAMPA = W(exc_index_1,neuron_flag,3);                   % => CON_temp = 0, initially the case
    end
    if ((E0_delayed > thresh) && (W(exc_index_1,neuron_flag,3) == 0))    %&& ((E0_before <= thresh) || (E0_before > thresh) ))  % new AP arrives and no residual conductance
        CON_temp_KAMPA = CON_Syn_exc_K_AMPA;                             % synaptic conductance CON_temp is full conductance after spike
        W(exc_index_1,neuron_flag,3) = CON_temp_KAMPA;                   % memorize for next timestep
        counters(neuron_flag,1) = 0;
    end
    
    I_Syn_exc_KAMPA                = weight * (E_Syn_exc_K_AMPA - Ecomp2) * CON_temp_KAMPA;
    I_Syn_exc_NMDA                 = weight * (E_Syn_exc_NMDA - Esoma) * CON_temp_NMDA * MgBlock;
    ISbuffer(lastbuff,exc_index_1) = 0;
    currents(neuron_flag,3)        = I_Syn_exc_KAMPA;
    currents(neuron_flag,4)        = I_Syn_exc_NMDA;
    temp(3,1)                      = I_Syn_exc_KAMPA;
    temp(4,1)                      = I_Syn_exc_NMDA;
    
end



%% --- 2nd excitatory input --- %%
%
% if (exc_index_2 ~= 0)
%
%     weight          = W(exc_index_2,neuron_flag,1);
%     E0_delayed      = ISbuffer(1/dt+1,exc_index_2);
%     E0_before       = ISbuffer(1/dt+2,exc_index_2);
%
%     if ((E0_delayed <= thresh) && (W(exc_index_2,neuron_flag,6) ~= 0))                                      % no more AP and no new AP, only residual conductance
%         CON_temp_NMDA = W(exc_index_2,neuron_flag,6) * exp(-(dt/tau_decay_exc_NMDA));                       % conductance decays further
%         W(exc_index_2,neuron_flag,6)  = CON_temp_NMDA;                                                      % memorize for next timestep
%     end
%     if ((E0_delayed > thresh) && (W(exc_index_2,neuron_flag,6) ~= 0) && (E0_before > thresh))               % within AP
%         CON_temp_NMDA = W(exc_index_2,neuron_flag,6) * exp(-(dt/tau_decay_exc_NMDA));                       % conductance decays further
%         W(exc_index_2,neuron_flag,6)  = CON_temp_NMDA;                                                      % memorize for next timestep
%     end
%     if ((E0_delayed > thresh) && (W(exc_index_2,neuron_flag,6) ~= 0) && (E0_before <= thresh))              % no more AP, new AP arrives and residual conductance is present
%         CON_temp_NMDA = CON_Syn_exc_NMDA;% +  W(exc_index_2,neuron_flag,6) * exp(-(dt/tau_decay_exc_NMDA));   % new conductance upon spike + residual decayed conductance
%         W(exc_index_2,neuron_flag,6)  = CON_temp_NMDA;                                                      % memorize for next timestep
%     end
%     if ((E0_delayed <= thresh) && (W(exc_index_2,neuron_flag,6) == 0))   % no AP and no residual conductance - ATTENTION: the == 0 if must come after the ~=, otherwise two ifs are evaluated
%         CON_temp_NMDA = W(exc_index_2,neuron_flag,6);                    % => CON_temp = 0, initially the case
%     end
%     if ((E0_delayed > thresh) && (W(exc_index_2,neuron_flag,6) == 0))    %&& ((E0_before <= thresh) || (E0_before > thresh) ))  % new AP arrives and no residual conductance
%         CON_temp_NMDA = CON_Syn_exc_NMDA;                                % synaptic conductance CON_temp is full conductance after spike
%         W(exc_index_2,neuron_flag,6) = CON_temp_NMDA;                    % memorize for next timestep
%     end
%
%     if ((E0_delayed <= thresh) && (W(exc_index_2,neuron_flag,5) ~= 0))                                           % no more AP and no new AP, only residual conductance
%         CON_temp_KAMPA = W(exc_index_2,neuron_flag,5) * exp(-(dt/tau_decay_exc_K_AMPA));                         % conductance decays further
%         W(exc_index_2,neuron_flag,5)  = CON_temp_KAMPA;                                                          % memorize for next timestep
%     end
%     if ((E0_delayed > thresh) && (W(exc_index_2,neuron_flag,5) ~= 0) &&  (E0_before > thresh))                   % within AP
%         CON_temp_KAMPA = W(exc_index_2,neuron_flag,5) * exp(-(dt/tau_decay_exc_K_AMPA));                         % conductance decays further
%         W(exc_index_2,neuron_flag,5)  = CON_temp_KAMPA;                                                          % memorize for next timestep
%     end
%     if ((E0_delayed > thresh) && (W(exc_index_2,neuron_flag,5) ~= 0) && (E0_before <= thresh))                   % no more AP, new AP arrives and residual conductance is present
%         if (counters(neuron_flag,2) == 0)
%             CON_temp_KAMPA =  W(exc_index_2,neuron_flag,5) * exp(-(dt/tau_decay_exc_K_AMPA));   % new conductance upon spike + residual decayed conductance
%             counters(neuron_flag,2) = 1;
%         else
%             CON_temp_KAMPA = CON_Syn_exc_K_AMPA +  W(exc_index_2,neuron_flag,5) * exp(-(dt/tau_decay_exc_K_AMPA));   % new conductance upon spike + residual decayed conductance
%         end
%         W(exc_index_2,neuron_flag,5)  = CON_temp_KAMPA;                                                          % memorize for next timestep
%     end
%     if ((E0_delayed <= thresh) && (W(exc_index_2,neuron_flag,5) == 0))   % no AP and no residual conductance
%         CON_temp_KAMPA = W(exc_index_2,neuron_flag,5);                   % => CON_temp = 0, initially the case
%     end
%     if ((E0_delayed > thresh) && (W(exc_index_2,neuron_flag,5) == 0))    %&& ((E0_before <= thresh) || (E0_before > thresh) ))  % new AP arrives and no residual conductance
%         CON_temp_KAMPA = CON_Syn_exc_K_AMPA;                             % synaptic conductance CON_temp is full conductance after spike
%         W(exc_index_2,neuron_flag,5) = CON_temp_KAMPA;                   % memorize for next timestep
%         counters(neuron_flag,2) = 0;
%     end
%
%     I_Syn_exc_KAMPA                = weight * (E_Syn_exc_K_AMPA - Ecomp2) * CON_temp_KAMPA;
%     I_Syn_exc_NMDA                 = weight * (E_Syn_exc_NMDA - Esoma) * CON_temp_NMDA * MgBlock;
%     ISbuffer(lastbuff,exc_index_2) = 0;
%     currents(neuron_flag,5)        = I_Syn_exc_KAMPA;
%     currents(neuron_flag,6)        = I_Syn_exc_NMDA;
%     temp(5,1)                      = I_Syn_exc_KAMPA;
%     temp(6,1)                      = I_Syn_exc_NMDA;
%
% end



%% --- Poisson Process Noise --- %%%

if network ~= 0
    p = poisspdf(1,0.3*dt);          % 300 events per sec means 0.3 events per ms means 0.3*dt events per time step
    a = rand;
    
    if (((a <= p)||(counters(neuron_flag,7) > 0)) || ((a <= p)&&(counters(neuron_flag,7) > 0)))
        if ( (counters(neuron_flag,7) > 0) && (W(1,neuron_flag,7) ~= 0) && (a > p) )      % within noise event, no new event
            CON_temp_KAMPA_N = W(1,neuron_flag,7) * exp(-(dt/tau_decay_exc_K_AMPA));      % conductance decays further
            W(1,neuron_flag,7)  = CON_temp_KAMPA_N;                                       % memorize for next timestep
            counters(neuron_flag,7) = counters(neuron_flag,7) + 1;                        % increment counter for noise event
        end
        if ((a <= p) && (counters(neuron_flag,7) > 0))
            CON_temp_KAMPA_N = CON_Syn_exc_K_AMPA + W(1,neuron_flag,7);                   % new noise event upon existing one
            W(1,neuron_flag,7)  = CON_temp_KAMPA_N;                                       % memorize for next timestep
            counters(neuron_flag,7) = 0;
            counters(neuron_flag,7) = counters(neuron_flag,7) + 1;
        end
        if ((counters(neuron_flag,7) == 0) && (W(1,neuron_flag,7) == 0) && (a <= p))      % new noise event arrives and no residual conductance
            CON_temp_KAMPA_N = CON_Syn_exc_K_AMPA;                                        % synaptic conductance CON_temp is full conductance after spike
            W(1,neuron_flag,7) = CON_temp_KAMPA_N;                                        % memorize for next timestep
            counters(neuron_flag,7) = counters(neuron_flag,7) + 1;                        % increment counter
        end
        
        I_noise                        = noiseAMP * (E_Syn_exc_K_AMPA - Ecomp1) * CON_temp_KAMPA_N;
        currents(neuron_flag,7)        = I_noise;
        temp(7,1)                      = I_noise;
    end
    
end

end