
function [] = callnetwork(Iin,NMDA,KAMPA,fiveHT,TTX,timeMS,netw)


%% inputs

% Iin:    initial stimulation current
% NMDA:   NMDA bath stimulation (sensible values )
% KAMPA:  AMPA bath stimulation (sensible values 0
% fiveHT: level of 5HT/serotonine, deteremines strength of afterhyperpolarization (0-1)
% TTX:    set to 1 to have TTX-like stimulation, suppresses action potential generation
% timeMS: simulation time in ms
% netw:   a flag which determines whether we simultate a single neuron (0) or the whole lamprey network (1)



%% --- description --- %%
%
% This set of MatLab scripts is part of the chapter: "Motor pattern generation",
% in "From Neuron to Cognition",  M.A. Arbib editor, MIT Press.
%
%
% Model implemented and documented by Andrej Bicanski, 
% implemented at Biorobotics laboratory,
% École polytechnique fédérale de Lausanne (EPFL)
% Currently at University College London, Institute of Cognitive
% Neuroscience (2015)
%
% For comments please email auke.ijspeert@epfl.ch or andrej.bicanski@gmail.com
%
% This main script sets up two hemi-segments, each consisting of 3 HH
% neurons, each representing a population: excitatory interneurons (EIN),
% cross-inhibitory interneurons (CCIN, inhibiting the other semi-segment),
% lateral interneurons (LIN).
%
% The network generates rythmic activity, alternating between the
% hemi-segments of the lamprey spinal segment. The neuons are modelled as 5
% compartment Hodgkin-Huxley neurons with multiple ion channels. The
% properties of the ion channels are crucial to the behavior of the network,
% since properies like adaptation (summed afterhyperpolarization) and the
% NMDA receptor activated synaptic channel properties determine the timing
% of firing between the neurons.
%
% other files:
% HHneuron_Wallen1992.m
% Syn_Trans.m
% implicit_solver.m
% ODEsolver.m
% plotting.m
%
%
% NOTE!
% The aim of this matlab implementation is not to be as
% computationally efficient as possible. Rather it aims to be as
% educational as possible. Students using these scripts should be
% able to follow all steps and concepts. Specific calculations could
% be implemented in a more efficient way.
%
%
%
% Main references:
%
% Ekeberg Ö, Wallén P, Lansner A, Tråvén H, Brodin L, Grillner S.
% A computer based model for realistic simulations of neural networks. I.
% The single neuron and synaptic interactions.
% Biol Cybern 65: 81–90, 1991.
%
% Wallén P, Ekeberg Ö, Lansner A, Brodin L, Tråvén H, Grillner S.
% A computer-based model for realistic simulations of neural networks. II.
% The segmental network generating locomotor rhythmicity in the lamprey.
% J Neurophysiol 68: 1939–1950, 1992.
%



%% --- runtime variables --- %%
%
Y_0          = zeros(16,1);            % for initial values of state variables for all neurons
t            = 0;                      % integer variable to count time in ms

if netw ~= 0
    dt = 0.1;
else
    dt           = 0.05;               % timestep in ms
end
lastbuff     = 10/dt;                  % number of elements necessary to represent 10 ms buffer
nodt         = timeMS/dt;              % number of timesteps
onesec       = 0;                      % number of timesteps to run before recording data
nodt2        = nodt+onesec;            % let the system run for 1 sec before measuring/recording
I            = zeros(8,13);            % currents for each neuron    (Dim 2 = inh, exc. K/AMPA, exc. NMDA and last dim for noise + additional kampa currents of other compartments)
temp         = zeros(7,1);             % for synaptic currents
counters     = zeros(8,7);             % to keep track of action potentials
cter         = zeros(8,2);             % for synaptic noise
Buffer       = zeros(lastbuff,8)-70;   % ...

data         = zeros(48,nodt,8);       % to keep track of data we might
% want, dimensionality: ( #of veriables , time , #of cells in network)
% variables = 12 state vars + 4 channel currents (Na, K, Ca, KdepCa) +
% 3 conductances (G_KdepCa, G_K, G_Na) + 3 synaptic currents, + 2 Ca
% pools + 3 chemical agents: Kainate, 5-HT and NMDA concetration + ...
% see plotting script

% Keeping track of data this was is not sensible with bigger networks and long simulation data.
% matlab would run out of memory. But here it serves so that we can look at
% all the variables


network = netw; % flag for single neuron (0) vs. network simulation (1)



%% --- connection matrix and relation matrix --- %%

W = zeros(8,8,8);            % matrix of weights, rows/columns: EIN_L, LIN_L, CCIN_L, MN_L, EIN_R, LIN_R, CCIN_R, MN_R
% Note, in this instance of the model no motoneurons are modeled (MNs). The corresponding
% matrix entries are empty. L=left, R=right
% 3rd dim: holds memory of previous conductance values (inh1, inh2, exc_KAMPA, exc_NMDA, noise) for corresponding synapse to this layer

if network ~= 0
    crossInhCoup = 0;
    W(1,1,1) =   0.4*8.0;        % EIN_L -> EIN_L   -  self excitatory  = W(sending,receiving,isweight)
    W(1,3,1) =   1*6.0;          % EIN_L -> CCIN_L  -  excitatory
    W(1,2,1) =   5.0*1.0;        % EIN_L -> LIN_L   -  excitatory 4
    W(2,3,1) =   2*1.0;          % LIN_L -> CCIN_L  -  inhibitory 2
    W(3,5,1) =   crossInhCoup;   % CCIN_L -> EIN_R  -  inhibitory
    W(3,6,1) =   crossInhCoup;   % CCIN_L -> LIN_R  -  inhibitory
    W(3,7,1) =   crossInhCoup;   % CCIN_L -> CCIN_R -  inhibitory
    W(3,8,1) =   crossInhCoup;   % CCIN_L -> MN_R   -  inhibitory
    W(5,5,1) =   0.4*8.0;        % EIN_R -> EIN_R   -  self excitatory
    W(5,7,1) =   1*6.0;          % EIN_R -> CCIN_R  -  excitatory
    W(5,6,1) =   5.0*1.0;        % EIN_R -> LIN_R   -  excitatory  4
    W(6,7,1) =   2*1.0;          % LIN_R -> CCIN_R  -  inhibitory  2
    W(7,1,1) =   crossInhCoup;   % CCIN_R -> EIN_L  -  inhibitory
    W(7,2,1) =   crossInhCoup;   % CCIN_R -> LIN_L  -  inhibitory
    W(7,3,1) =   crossInhCoup;   % CCIN_R -> CCIN_L -  inhibitory
    W(7,4,1) =   crossInhCoup;   % CCIN_R -> MN_L   -  inhibitory
end
Rel = zeros(2,2,8);               % Matrix of contact relations. 3D array. 1st two dimensions give the flags of excitatory (column 1) and inhib. (col. 2) input neurons
% for neuron x, where x is the value of the 3rd dim. E.g. layer 1 (... , ... , 1) has all the input relations for neuron 1, the left EIN
% this is used to automate the selction of the correct inputs for the synaptic transmission routine
% Dim.1=2 for inh. and exc., Dim.2 becuase we have max 2 inh. or exc. inputs per cell, Dim.3.=8 for 8 neuron types

Rel(1,1,1) = 1; Rel(1,2,1) = 7;   % meaning neuron one gets excitatry input from neuron 1 and inhibitory input from neuron 7
Rel(1,1,2) = 1; Rel(1,2,2) = 7;
Rel(1,1,3) = 1; Rel(1,2,3) = 2; Rel(2,2,3) = 7;
Rel(1,1,4) = 1; Rel(1,2,4) = 7;
Rel(1,1,5) = 5; Rel(1,2,5) = 3;
Rel(1,1,6) = 5; Rel(1,2,6) = 3;
Rel(1,1,7) = 5; Rel(1,2,7) = 6; Rel(2,2,7) = 3;
Rel(1,1,8) = 4; Rel(1,2,8) = 3;



%% --- preallocation and inital values --- %%

coeffis       = zeros(16,2);            % coefficients of DEQs of type dY/dt = K+LY, 1st column K, 2nd column L
statevars     = zeros(16,2);            % 1st column contains updated variables, 2nd old ones

restingPot    = -70;                    %

Y_0(12,1)     = restingPot;             % initial segment initial voltage
Y_0(1,1)      = restingPot;             % Esoma inital value = resting potential
Y_0(2,1)      = restingPot;             % Ecom1 inital value = resting potential
Y_0(3,1)      = restingPot;             % Ecom2 inital value = resting potential
Y_0(4,1)      = restingPot;             % Ecom3 inital value = resting potential
Y_0(5,1)      = 0;                      % m_0 initial activation of Na channels
Y_0(6,1)      = 1;                      % h_0 initial inactivation of Na channels start 1
Y_0(7,1)      = 0;                      % n_0 initial activation of K channels
Y_0(8,1)      = 0;                      % q_0 initial activation of Ca channels
Y_0(9,1)      = 0;                      % CaAP_0 initial relative Ca concentration for K
Y_0(10,1)     = 0;                      % p fraction of nmda channles in open state
Y_0(11,1)     = 0;                      % CaNMDA_0 initial relative Ca concentration due tu Calcium entering the NMDA synapse
Y_0(13,1)     = 0;                      %
Y_0(14,1)     = 1;                      %
Y_0(15,1)     = 0;                      %

Yold_EIN_L    = Y_0;                    % initial values for all network cells
Yold_LIN_L    = Y_0;                    % ...
Yold_CCIN_L   = Y_0;
Yold_EIN_R    = Y_0;
Yold_LIN_R    = Y_0;
Yold_CCIN_R   = Y_0;


% Na_m_ALPH_A   =    0.2;   % A coefficient   [1/mV 1/ms]
% Na_m_ALPH_B   =    -45;   % B coefficient   [mV]
% Na_m_ALPH_C   =      1;   % C coefficient   [mV]
% Na_m_BETA_A   =   0.06;   % ...
% Na_m_BETA_B   =    -54;
% Na_m_BETA_C   =     20;
% Na_h_ALPH_A   =   0.08;
% Na_h_ALPH_B   =    -45;
% Na_h_ALPH_C   =      1;
% Na_h_BETA_A   =    0.4;
% Na_h_BETA_B   =    -41;
% Na_h_BETA_C   =      2;
% K_n_ALPH_A    =   0.02;
% K_n_ALPH_B    =    -45;
% K_n_ALPH_C    =    0.8;
% K_n_BETA_A    =  0.005;
% K_n_BETA_B    =    -35;
% K_n_BETA_C    =    0.4;
% Ca_q_ALPH_A   =   0.08;
% Ca_q_ALPH_B   =     15;
% Ca_q_ALPH_C   =     11;
% Ca_q_BETA_A   =  0.001;
% Ca_q_BETA_B   =     15;
% Ca_q_BETA_C   =    0.5;
% NMDA_p_ALPH_A =    0.7;
% NMDA_p_ALPH_C =     17;
% NMDA_p_MG     =    1.8;   % for bath application
% NMDA_p_BETA_A = 0.0056;   
% NMDA_p_BETA_C =     17;   
% 
% channelparas = zeros(29,1);
% 
% channelparas(1,1) = Na_m_ALPH_A;   % A coefficient   [1/mV 1/ms]
% channelparas(2,1) = Na_m_ALPH_B;   % B coefficient   [mV]
% channelparas(3,1) = Na_m_ALPH_C;   % C coefficient   [mV]
% channelparas(4,1) = Na_m_BETA_A;   % ...
% channelparas(5,1) = Na_m_BETA_B;
% channelparas(6,1) = Na_m_BETA_C;
% channelparas(7,1) = Na_h_ALPH_A;
% channelparas(8,1) = Na_h_ALPH_B;
% channelparas(9,1) = Na_h_ALPH_C;
% channelparas(10,1) = Na_h_BETA_A;
% channelparas(11,1) = Na_h_BETA_B;
% channelparas(12,1) = Na_h_BETA_C;
% channelparas(13,1) = K_n_ALPH_A;
% channelparas(14,1) = K_n_ALPH_B;
% channelparas(15,1) = K_n_ALPH_C;
% channelparas(16,1) = K_n_BETA_A;
% channelparas(17,1) = K_n_BETA_B;
% channelparas(18,1) = K_n_BETA_C;
% channelparas(19,1) = Ca_q_ALPH_A;
% channelparas(20,1) = Ca_q_ALPH_B;
% channelparas(21,1) = Ca_q_ALPH_C;
% channelparas(22,1) = Ca_q_BETA_A;
% channelparas(23,1) = Ca_q_BETA_B;
% channelparas(24,1) = Ca_q_BETA_C;
% channelparas(25,1) = NMDA_p_ALPH_A;
% channelparas(26,1) = NMDA_p_ALPH_C;
% channelparas(27,1) = NMDA_p_MG; 
% channelparas(28,1) = NMDA_p_BETA_A;   
% channelparas(29,1) = NMDA_p_BETA_C;   



%% --- run parameters --- %%

nmda          = NMDA;                   % NMDA concentration in the bath
kampa         = KAMPA;                  % AMPA concentration in the bath
KAHP          = 1.0;%*0.9;              % strenght of the fast Ca-dependent K-current AHP
fiveHT        = fiveHT;                 %
ttx           = TTX;                    % Channel block due to TTX - set to 1 to block Na => no APs
noiseAMP      = 0.02;                   % noise amplitude 0.2
gCa           = 0;                      % Calcium current conductance
I_init        = Iin;                    % initial stimulation
a             = 2*500/dt;               % start index of stim on left  side
b             = 2*1500/dt;              % end   index of stim on right side
solver        = 1;                      % choose ODE solver: 1=exponential Euler, 2=RK4 ... more to come
stimstring    = 'StimulationPattern';   % specify a descriptive string to be included in the filename of the saved datafile



%% --- main loop --- %%

for i=1:2*nodt2
    
    if (i > a) && (i < b) % inject current between timestep a and timestep b
        Iinit = I_init;
    else
        Iinit = 0;
    end
    
    [Ynew_EIN_L, data1,Buffer,W,counters,cter,I,temp] = HHneuron_Wallen1992(Yold_EIN_L, dt,Iinit,i,1,Buffer,W,counters,cter,I,Rel,lastbuff,temp,nmda,kampa,fiveHT,ttx,noiseAMP,gCa,solver,t,KAHP,statevars,coeffis,network);    % left EIN
   
    if network ~= 0
        [Ynew_LIN_L, data2,Buffer,W,counters,cter,I,temp] = HHneuron_Wallen1992(Yold_LIN_L, dt,Iinit,i,2,Buffer,W,counters,cter,I,Rel,lastbuff,temp,nmda,kampa,fiveHT,ttx,noiseAMP,gCa,solver,t,KAHP,statevars,coeffis,network);    % left LIN
        [Ynew_CCIN_L,data3,Buffer,W,counters,cter,I,temp] = HHneuron_Wallen1992(Yold_CCIN_L,dt,Iinit,i,3,Buffer,W,counters,cter,I,Rel,lastbuff,temp,nmda,kampa,fiveHT,ttx,noiseAMP,gCa,solver,t,KAHP,statevars,coeffis,network);    % left CCIN
        [Ynew_EIN_R, data5,Buffer,W,counters,cter,I,temp] = HHneuron_Wallen1992(Yold_EIN_R, dt,Iinit,i,5,Buffer,W,counters,cter,I,Rel,lastbuff,temp,nmda,kampa,fiveHT,ttx,noiseAMP,gCa,solver,t,KAHP,statevars,coeffis,network);    % right EIN
        [Ynew_LIN_R, data6,Buffer,W,counters,cter,I,temp] = HHneuron_Wallen1992(Yold_LIN_R, dt,Iinit,i,6,Buffer,W,counters,cter,I,Rel,lastbuff,temp,nmda,kampa,fiveHT,ttx,noiseAMP,gCa,solver,t,KAHP,statevars,coeffis,network);    % right LIN
        [Ynew_CCIN_R,data7,Buffer,W,counters,cter,I,temp] = HHneuron_Wallen1992(Yold_CCIN_R,dt,Iinit,i,7,Buffer,W,counters,cter,I,Rel,lastbuff,temp,nmda,kampa,fiveHT,ttx,noiseAMP,gCa,solver,t,KAHP,statevars,coeffis,network);    % right CCIN
    end
    
    if (mod(i,2) == 0)
        if (i >= 2*onesec)
            data(:,i/2-onesec+1,1)  = data1;
            if network ~= 0
                data(:,i/2-onesec+1,2)  = data2;
                data(:,i/2-onesec+1,3)  = data3;
                data(:,i/2-onesec+1,5)  = data5;
                data(:,i/2-onesec+1,6)  = data6;
                data(:,i/2-onesec+1,7)  = data7;
            end
            data(47,i/2-onesec+1,1) = t;
        end
        Buffer = circshift(Buffer,1);
    end
    
    Yold_EIN_L  =  Ynew_EIN_L;
    if network ~= 0
        Yold_LIN_L  =  Ynew_LIN_L;
        Yold_CCIN_L = Ynew_CCIN_L;
        Yold_EIN_R  =  Ynew_EIN_R;
        Yold_LIN_R  =  Ynew_LIN_R;
        Yold_CCIN_R = Ynew_CCIN_R;
    end
    t=t+dt/2;
    
    if (i == 2*nodt2)
        'You are done.'
    end
    
end



%% --- saving and plotting --- %%

if network ~= 0
    mystring = ['networkdata_dt' num2str(dt) '_#' int2str(timeMS) '_NMDA' num2str(nmda) '_kampa' num2str(kampa) '_TTX' int2str(TTX) '_fiveHT' num2str(fiveHT) '_Iinit' ...
        num2str(I_init) '_noiseAMP' num2str(noiseAMP) '_' stimstring '.mat'];
    save(mystring, 'data', '-v7.3');
    plotting(dt,timeMS,nmda,kampa,TTX,fiveHT,I_init,noiseAMP,stimstring);
end

end
